@@ -99,7 +99,40 @@ impl<T> Default for MaybeUndefined<T> {
     }
 }
 
+impl<T> From<T> for MaybeUndefined<T> {
+    fn from(value: T) -> Self {
+        MaybeUndefined::Value(value)
+    }
+}
+
+impl<T> IntoIterator for MaybeUndefined<T> {
+    type Item = T;
+    type IntoIter = std::option::IntoIter<T>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        self.take().into_iter()
+    }
+}
+
 impl<T> MaybeUndefined<T> {
+    /// Create a `MaybeUndefined<T>` from `Option<T>`, returns
+    /// `MaybeUndefined::Undefined` if this value is none.
+    pub fn from_opt_undefined(value: Option<T>) -> Self {
+        match value {
+            Some(value) => MaybeUndefined::Value(value),
+            None => MaybeUndefined::Undefined,
+        }
+    }
+
+    /// Create a `MaybeUndefined<T>` from `Option<T>`, returns
+    /// `MaybeUndefined::Null` if this value is none.
+    pub fn from_opt_null(value: Option<T>) -> Self {
+        match value {
+            Some(value) => MaybeUndefined::Value(value),
+            None => MaybeUndefined::Null,
+        }
+    }
+
     /// Returns true if the `MaybeUndefined<T>` is undefined.
     #[inline]
     pub const fn is_undefined(&self) -> bool {
@@ -118,8 +151,8 @@ impl<T> MaybeUndefined<T> {
         matches!(self, MaybeUndefined::Value(_))
     }
 
-    /// Borrow the value, returns `None` if the the `MaybeUndefined<T>` is
-    /// `undefined` or `null`, otherwise returns `Some(T)`.
+    /// Returns `None` if the the `MaybeUndefined<T>` is
+    /// `undefined` or `null`, otherwise returns `Some(&T)`.
     #[inline]
     pub const fn value(&self) -> Option<&T> {
         match self {
@@ -128,6 +161,16 @@ impl<T> MaybeUndefined<T> {
         }
     }
 
+    /// Returns `None` if the the `MaybeUndefined<T>` is
+    /// `undefined` or `null`, otherwise returns `Some(&mut T)`.
+    #[inline]
+    pub fn value_mut(&mut self) -> Option<&mut T> {
+        match self {
+            MaybeUndefined::Value(value) => Some(value),
+            _ => None,
+        }
+    }
+
     /// Converts the `MaybeUndefined<T>` to `Option<T>`.
     #[inline]
     pub fn take(self) -> Option<T> {
@@ -137,6 +180,16 @@ impl<T> MaybeUndefined<T> {
         }
     }
 
+    /// Converts from `&MaybeUndefined<T>` to `MaybeUndefined<&T>`.
+    #[inline]
+    pub const fn as_ref(&self) -> MaybeUndefined<&T> {
+        match self {
+            MaybeUndefined::Undefined => MaybeUndefined::Undefined,
+            MaybeUndefined::Null => MaybeUndefined::Null,
+            MaybeUndefined::Value(value) => MaybeUndefined::Value(value),
+        }
+    }
+
     /// Converts the `MaybeUndefined<T>` to `Option<Option<T>>`.
     #[inline]
     pub const fn as_opt_ref(&self) -> Option<Option<&T>> {
@@ -216,6 +269,19 @@ impl<T> MaybeUndefined<T> {
     }
 }
 
+impl<T: Deref> MaybeUndefined<T> {
+    /// Converts from `MaybeUndefined<T>` (or `&MaybeUndefined<T>`) to
+    /// `MaybeUndefined<&T::Target>`.
+    #[inline]
+    pub fn as_deref(&self) -> MaybeUndefined<&T::Target> {
+        match self {
+            MaybeUndefined::Undefined => MaybeUndefined::Undefined,
+            MaybeUndefined::Null => MaybeUndefined::Null,
+            MaybeUndefined::Value(value) => MaybeUndefined::Value(value.deref()),
+        }
+    }
+}
+
 impl<T: Type> Type for MaybeUndefined<T> {
     const IS_REQUIRED: bool = false;
 
